/// <reference types="react" />
import * as React from 'react';
import { Interval } from '../core';
import { ChartId, SeriesId, TBySeriesId, DataLoader, DebugStoreHooks } from './interfaces';
import { DefaultChartState } from './model/state';
export interface Props {
    seriesIds: SeriesId[];
    loadData: DataLoader;
    className?: string;
    pixelRatio?: number;
    chartId?: ChartId;
    defaultState?: DefaultChartState;
    onLoadStateChange?: (isLoading: TBySeriesId<boolean>) => void;
    onError?: (errors: TBySeriesId<any>) => void;
    includeResizeSentinel?: boolean;
    loadDataContext?: any;
    loadDataDebounceTimeout?: number;
    debugStoreHooks?: DebugStoreHooks;
    xDomain?: Interval;
    onXDomainChange?: (xDomain: Interval) => void;
    yDomains?: TBySeriesId<Interval>;
    onYDomainsChange?: (yDomains: TBySeriesId<Interval>) => void;
    selection?: Interval | 'none';
    onSelectionChange?: (selection: Interval) => void;
    hover?: number | 'none';
    onHoverChange?: (hover: number) => void;
}
export default class ChartProvider extends React.PureComponent<Props, void> {
    static propTypes: React.ValidationMap<Props>;
    private _store;
    private _lastState;
    private _unsubscribeCallback;
    static defaultProps: Partial<Props>;
    componentWillMount(): void;
    componentWillReceiveProps(nextProps: Props): void;
    componentWillUnmount(): void;
    private _onStoreChange(props);
    private _maybeFireAllCallbacks();
    private _maybeFireCallback<T>(currentState, lastState, callback?);
    private _onPropsChange(nextProps);
    private _maybeDispatchChangedProp<T>(prop, nextProp, actionCreator);
    render(): JSX.Element;
}
