"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var React = require("react");
var classNames = require("classnames");
var react_redux_1 = require("react-redux");
var core_1 = require("../core");
var storeFactory_1 = require("./flux/storeFactory");
var ConnectedResizeSentinelLayer_1 = require("./layers/ConnectedResizeSentinelLayer");
var atomicActions_1 = require("./flux/atomicActions");
var compoundActions_1 = require("./flux/compoundActions");
var ChartProvider = (function (_super) {
    __extends(ChartProvider, _super);
    function ChartProvider() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    ChartProvider.prototype.componentWillMount = function () {
        this._store = storeFactory_1.default(this.props.chartId, this.props.debugStoreHooks);
        this._onStoreChange(this.props);
    };
    ChartProvider.prototype.componentWillReceiveProps = function (nextProps) {
        if (nextProps.chartId !== this.props.chartId) {
            this._unsubscribeCallback();
            this._store = storeFactory_1.default(this.props.chartId, this.props.debugStoreHooks);
            this._onStoreChange(nextProps);
        }
        else {
            this._onPropsChange(nextProps);
        }
    };
    ChartProvider.prototype.componentWillUnmount = function () {
        this._unsubscribeCallback();
    };
    ChartProvider.prototype._onStoreChange = function (props) {
        this._lastState = this._store.getState();
        this._store.dispatch(compoundActions_1.setSeriesIdsAndLoad(props.seriesIds));
        this._store.dispatch(compoundActions_1.setDataLoaderAndLoad(props.loadData));
        this._store.dispatch(compoundActions_1.setDataLoaderContextAndLoad(props.loadDataContext));
        if (_.isNumber(props.loadDataDebounceTimeout)) {
            this._store.dispatch(atomicActions_1.setDataLoaderDebounceTimeout(props.loadDataDebounceTimeout));
        }
        // These should perhaps be set on the store as explicit "default" fields rather than auto-dispatched on load.
        if (props.xDomain) {
            this._store.dispatch(compoundActions_1.setOverrideXDomainAndLoad(props.xDomain));
        }
        else if (props.defaultState && props.defaultState.xDomain) {
            this._store.dispatch(compoundActions_1.setXDomainAndLoad(props.defaultState.xDomain));
        }
        if (props.yDomains) {
            this._store.dispatch(atomicActions_1.setOverrideYDomains(props.yDomains));
        }
        else if (props.defaultState && props.defaultState.yDomains) {
            this._store.dispatch(atomicActions_1.setYDomains(props.defaultState.yDomains));
        }
        if (props.hover != null) {
            this._store.dispatch(atomicActions_1.setOverrideHover(props.hover));
        }
        if (props.selection) {
            this._store.dispatch(atomicActions_1.setOverrideSelection(props.selection));
        }
        this._maybeFireAllCallbacks();
        this._unsubscribeCallback = this._store.subscribe(this._maybeFireAllCallbacks.bind(this));
    };
    ChartProvider.prototype._maybeFireAllCallbacks = function () {
        var state = this._store.getState();
        this._maybeFireCallback(state.uiState.xDomain, this._lastState.uiState.xDomain, this.props.onXDomainChange);
        this._maybeFireCallback(state.uiState.selection, this._lastState.uiState.selection, this.props.onSelectionChange);
        this._maybeFireCallback(state.uiState.hover, this._lastState.uiState.hover, this.props.onHoverChange);
        this._maybeFireCallback(state.errorBySeriesId, this._lastState.errorBySeriesId, this.props.onError);
        if (this.props.onLoadStateChange) {
            if (state.loadVersionBySeriesId !== this._lastState.loadVersionBySeriesId) {
                this.props.onLoadStateChange(_.mapValues(state.loadVersionBySeriesId, function (v) { return !!v; }));
            }
        }
        if (this.props.onYDomainsChange) {
            if (state.uiState.yDomainBySeriesId !== this._lastState.uiState.yDomainBySeriesId ||
                state.loadedDataBySeriesId !== this._lastState.loadedDataBySeriesId) {
                var internalYDomains = _.assign(_.mapValues(state.loadedDataBySeriesId, function (loadedData) { return loadedData.yDomain; }), state.uiState.yDomainBySeriesId);
                this.props.onYDomainsChange(internalYDomains);
            }
        }
        this._lastState = state;
    };
    ChartProvider.prototype._maybeFireCallback = function (currentState, lastState, callback) {
        if (callback && currentState !== lastState) {
            callback(currentState);
        }
    };
    ChartProvider.prototype._onPropsChange = function (nextProps) {
        this._maybeDispatchChangedProp(this.props.seriesIds, nextProps.seriesIds, compoundActions_1.setSeriesIdsAndLoad);
        this._maybeDispatchChangedProp(this.props.loadData, nextProps.loadData, compoundActions_1.setDataLoaderAndLoad);
        this._maybeDispatchChangedProp(this.props.loadDataContext, nextProps.loadDataContext, compoundActions_1.setDataLoaderContextAndLoad);
        this._maybeDispatchChangedProp(this.props.loadDataDebounceTimeout, nextProps.loadDataDebounceTimeout, atomicActions_1.setDataLoaderDebounceTimeout);
        this._maybeDispatchChangedProp(this.props.xDomain, nextProps.xDomain, compoundActions_1.setOverrideXDomainAndLoad);
        this._maybeDispatchChangedProp(this.props.yDomains, nextProps.yDomains, atomicActions_1.setOverrideYDomains);
        this._maybeDispatchChangedProp(this.props.hover, nextProps.hover, atomicActions_1.setOverrideHover);
        this._maybeDispatchChangedProp(this.props.selection, nextProps.selection, atomicActions_1.setOverrideSelection);
    };
    ChartProvider.prototype._maybeDispatchChangedProp = function (prop, nextProp, actionCreator) {
        if (prop !== nextProp) {
            this._store.dispatch(actionCreator(nextProp));
        }
    };
    ChartProvider.prototype.render = function () {
        return (React.createElement(react_redux_1.Provider, { store: this._store },
            React.createElement("div", { className: classNames('lc-chart-provider', this.props.className) },
                this.props.includeResizeSentinel
                    ? React.createElement(core_1.Stack, { className: 'autoinjected-resize-sentinel-stack' },
                        React.createElement(ConnectedResizeSentinelLayer_1.default, null))
                    : null,
                this.props.children)));
    };
    return ChartProvider;
}(React.PureComponent));
ChartProvider.propTypes = {
    seriesIds: React.PropTypes.arrayOf(React.PropTypes.string).isRequired,
    loadData: React.PropTypes.func.isRequired,
    className: React.PropTypes.string,
    pixelRatio: React.PropTypes.number,
    chartId: React.PropTypes.string,
    defaultState: core_1.propTypes.defaultChartState,
    onLoadStateChange: React.PropTypes.func,
    onError: React.PropTypes.func,
    includeResizeSentinel: React.PropTypes.bool,
    // loadDataContext doesn't need a prop type, as it's `any?` and therefore we can make no assertions about it.
    loadDataDebounceTimeout: React.PropTypes.number,
    xDomain: core_1.propTypes.interval,
    onXDomainChange: React.PropTypes.func,
    yDomains: React.PropTypes.objectOf(core_1.propTypes.interval),
    onYDomainsChange: React.PropTypes.func,
    selection: core_1.propTypes.controlledInterval,
    onSelectionChange: React.PropTypes.func,
    hover: core_1.propTypes.controlledHover,
    onHoverChange: React.PropTypes.func,
};
ChartProvider.defaultProps = {
    includeResizeSentinel: true
};
ChartProvider = __decorate([
    core_1.PixelRatioContextProvider
], ChartProvider);
exports.default = ChartProvider;
//# sourceMappingURL=ChartProvider.js.map