"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var reselect_1 = require("reselect");
var selectors_1 = require("../model/selectors");
var _unusedToMakeTscQuiet = null;
_unusedToMakeTscQuiet = _unusedToMakeTscQuiet;
function _castToOpaqueInput(selector) {
    return selector;
}
exports.selectXDomain = _castToOpaqueInput(selectors_1.selectXDomain);
exports.selectYDomains = _castToOpaqueInput(selectors_1.selectYDomains);
exports.selectHover = _castToOpaqueInput(selectors_1.selectHover);
exports.selectSelection = _castToOpaqueInput(selectors_1.selectSelection);
exports.selectData = _castToOpaqueInput(selectors_1.selectData);
exports.selectIsLoading = _castToOpaqueInput(function (state) { return _.mapValues(state.loadVersionBySeriesId, function (v) { return !!v; }); });
exports.selectErrors = _castToOpaqueInput(function (state) { return state.errorBySeriesId; });
exports.selectChartPixelWidth = _castToOpaqueInput(function (state) { return state.physicalChartWidth; });
// This is cause sortedIndexBy prefers to have the same shape for the array items and the searched thing. We don't
// know what that shape is, so we have a sentinel + accompanying function to figure out when it's asking for the hover value.
function HOVER_VALUE_SENTINEL() { }
function createSelectDataForHover(xValueSelector) {
    return _castToOpaqueInput(reselect_1.createSelector(selectors_1.selectData, selectors_1.selectHover, function (dataBySeriesId, hover) {
        if (_.isNil(hover)) {
            return _.mapValues(dataBySeriesId, _.constant(undefined));
        }
        else {
            var xIterator_1 = function (seriesId, datum) {
                return datum === HOVER_VALUE_SENTINEL ? hover : xValueSelector(seriesId, datum);
            };
            return _.mapValues(dataBySeriesId, function (data, seriesId) {
                // -1 because sortedIndexBy returns the first index that would be /after/ the input value, but we're trying to
                // get whichever value comes before. Note that this may return undefined, but that's specifically allowed:
                // there may not be an appropriate hover value for this series.
                return data[_.sortedIndexBy(data, HOVER_VALUE_SENTINEL, xIterator_1.bind(null, seriesId)) - 1];
            });
        }
    }));
}
exports.createSelectDataForHover = createSelectDataForHover;
//# sourceMappingURL=exportableSelectors.js.map