"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var selectors_1 = require("../model/selectors");
var atomicActions_1 = require("./atomicActions");
function setXDomainAndLoad(payload) {
    return function (dispatch, getState) {
        var state = getState();
        if (!_.isEqual(payload, state.uiState.xDomain)) {
            dispatch(atomicActions_1.setXDomain(payload));
            if (!state.uiStateConsumerOverrides.xDomain) {
                dispatch(_requestDataLoad());
            }
        }
    };
}
exports.setXDomainAndLoad = setXDomainAndLoad;
function setOverrideXDomainAndLoad(payload) {
    return function (dispatch, getState) {
        var state = getState();
        if (!_.isEqual(payload, state.uiStateConsumerOverrides.xDomain)) {
            dispatch(atomicActions_1.setOverrideXDomain(payload));
            dispatch(_requestDataLoad());
        }
    };
}
exports.setOverrideXDomainAndLoad = setOverrideXDomainAndLoad;
function setChartPhysicalWidthAndLoad(payload) {
    return function (dispatch, getState) {
        var state = getState();
        if (payload !== state.physicalChartWidth) {
            dispatch(atomicActions_1.setChartPhysicalWidth(payload));
            dispatch(_requestDataLoad());
        }
    };
}
exports.setChartPhysicalWidthAndLoad = setChartPhysicalWidthAndLoad;
function setSeriesIdsAndLoad(payload) {
    return function (dispatch, getState) {
        var state = getState();
        var orderedSeriesIds = _.sortBy(payload);
        if (!_.isEqual(orderedSeriesIds, state.seriesIds)) {
            var newSeriesIds = _.difference(orderedSeriesIds, state.seriesIds);
            dispatch(atomicActions_1.setSeriesIds(orderedSeriesIds));
            dispatch(_requestDataLoad(newSeriesIds));
        }
    };
}
exports.setSeriesIdsAndLoad = setSeriesIdsAndLoad;
function setDataLoaderAndLoad(payload) {
    return function (dispatch, getState) {
        var state = getState();
        if (state.dataLoader !== payload) {
            dispatch(atomicActions_1.setDataLoader(payload));
            dispatch(_requestDataLoad());
        }
    };
}
exports.setDataLoaderAndLoad = setDataLoaderAndLoad;
function setDataLoaderContextAndLoad(payload) {
    return function (dispatch, getState) {
        var state = getState();
        if (payload !== state.loaderContext) {
            dispatch(atomicActions_1.setDataLoaderContext(payload));
            dispatch(_requestDataLoad());
        }
    };
}
exports.setDataLoaderContextAndLoad = setDataLoaderContextAndLoad;
// Exported for testing.
function _requestDataLoad(seriesIds) {
    return function (dispatch, getState) {
        var existingSeriesIds = getState().seriesIds;
        var seriesIdsToLoad = seriesIds
            ? _.intersection(seriesIds, existingSeriesIds)
            : existingSeriesIds;
        dispatch(atomicActions_1.dataRequested(seriesIdsToLoad));
        dispatch(_performDataLoad());
    };
}
exports._requestDataLoad = _requestDataLoad;
// Exported for testing.
function _makeKeyedDataBatcher(onBatch, timeout) {
    var keyedBatchAccumulator = {};
    var throttledBatchCallback = _.throttle(function () {
        // Save it off first in case the batch triggers any more additions.
        var batchData = keyedBatchAccumulator;
        keyedBatchAccumulator = {};
        onBatch(batchData);
    }, timeout, { leading: false, trailing: true });
    return function (keyedData) {
        _.assign(keyedBatchAccumulator, keyedData);
        throttledBatchCallback();
    };
}
exports._makeKeyedDataBatcher = _makeKeyedDataBatcher;
// Exported for testing.
function _performDataLoad(batchingTimeout) {
    if (batchingTimeout === void 0) { batchingTimeout = 200; }
    return function (dispatch, getState) {
        var debounceTimeout = getState().debounceTimeout;
        // redux-debounced checks falsy-ness, so 0 will behave as if there is no debouncing!
        debounceTimeout = debounceTimeout === 0 ? 1 : debounceTimeout;
        var adjustedBatchingTimeout = Math.min(batchingTimeout, debounceTimeout);
        var thunk = function (dispatch, getState) {
            var preLoadChartState = getState();
            var dataLoader = preLoadChartState.dataLoader;
            var loaderContext = preLoadChartState.loaderContext;
            var seriesIdsToLoad = _.keys(_.pickBy(preLoadChartState.loadVersionBySeriesId));
            var loadPromiseBySeriesId = dataLoader(seriesIdsToLoad, selectors_1.selectXDomain(preLoadChartState), preLoadChartState.physicalChartWidth, preLoadChartState.loadedDataBySeriesId, loaderContext);
            var batchedDataReturned = _makeKeyedDataBatcher(function (payload) {
                dispatch(atomicActions_1.dataReturned(payload));
            }, adjustedBatchingTimeout);
            var batchedDataErrored = _makeKeyedDataBatcher(function (payload) {
                dispatch(atomicActions_1.dataErrored(payload));
            }, adjustedBatchingTimeout);
            function isResultStillRelevant(postLoadChartState, seriesId) {
                return preLoadChartState.loadVersionBySeriesId[seriesId] === postLoadChartState.loadVersionBySeriesId[seriesId];
            }
            var dataPromises = _.map(loadPromiseBySeriesId, function (dataPromise, seriesId) {
                return dataPromise
                    .then(function (loadedData) {
                    if (isResultStillRelevant(getState(), seriesId)) {
                        batchedDataReturned((_a = {},
                            _a[seriesId] = loadedData,
                            _a));
                    }
                    var _a;
                })
                    .catch(function (error) {
                    if (isResultStillRelevant(getState(), seriesId)) {
                        batchedDataErrored((_a = {},
                            _a[seriesId] = error,
                            _a));
                    }
                    var _a;
                });
            });
            return Promise.all(dataPromises);
        };
        thunk.meta = {
            debounce: {
                time: debounceTimeout,
                key: 'data-load'
            }
        };
        return dispatch(thunk);
    };
}
exports._performDataLoad = _performDataLoad;
//# sourceMappingURL=compoundActions.js.map