"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var immutability_helper_1 = require("immutability-helper");
var update = immutability_helper_1.newContext();
update.extend('$assign', function (spec, object) { return _.assign({}, object, spec); });
var atomicActions_1 = require("./atomicActions");
var state_1 = require("../model/state");
var constants_1 = require("../model/constants");
var reducerUtils_1 = require("./reducerUtils");
function default_1(state, action) {
    if (state === undefined) {
        return state_1.DEFAULT_CHART_STATE;
    }
    switch (action.type) {
        case atomicActions_1.ActionType.SET_SERIES_IDS: {
            var seriesIds = action.payload;
            return update(state, {
                seriesIds: { $set: seriesIds },
                loadedDataBySeriesId: { $set: reducerUtils_1.objectWithKeysFromObject(state.loadedDataBySeriesId, seriesIds, { data: [], yDomain: constants_1.DEFAULT_Y_DOMAIN }) },
                loadVersionBySeriesId: { $set: reducerUtils_1.objectWithKeysFromObject(state.loadVersionBySeriesId, seriesIds, null) },
                errorBySeriesId: { $set: reducerUtils_1.objectWithKeysFromObject(state.errorBySeriesId, seriesIds, null) }
            });
        }
        case atomicActions_1.ActionType.DATA_REQUESTED:
            return update(state, {
                loadVersionBySeriesId: { $assign: reducerUtils_1.objectWithKeys(action.payload, _.uniqueId('load-version-')) }
            });
        case atomicActions_1.ActionType.DATA_RETURNED:
            return update(state, {
                loadedDataBySeriesId: { $assign: action.payload },
                loadVersionBySeriesId: { $assign: reducerUtils_1.replaceValuesWithConstant(action.payload, null) },
                errorBySeriesId: { $assign: reducerUtils_1.replaceValuesWithConstant(action.payload, null) }
            });
        case atomicActions_1.ActionType.DATA_ERRORED:
            // TODO: Should we clear the current data too?
            return update(state, {
                loadVersionBySeriesId: { $assign: reducerUtils_1.replaceValuesWithConstant(action.payload, null) },
                errorBySeriesId: { $assign: action.payload }
            });
        case atomicActions_1.ActionType.SET_DATA_LOADER:
            return update(state, {
                dataLoader: { $set: action.payload || state_1.invalidLoader }
            });
        case atomicActions_1.ActionType.SET_DATA_LOADER_CONTEXT:
            return update(state, {
                loaderContext: { $set: action.payload }
            });
        case atomicActions_1.ActionType.SET_DATA_LOADER_DEBOUNCE_TIMEOUT:
            return update(state, {
                debounceTimeout: { $set: _.isNumber(action.payload) ? action.payload : state.debounceTimeout }
            });
        case atomicActions_1.ActionType.SET_CHART_PHYSICAL_WIDTH:
            return update(state, {
                physicalChartWidth: { $set: action.payload }
            });
        case atomicActions_1.ActionType.SET_X_DOMAIN:
            return update(state, {
                uiState: {
                    xDomain: { $set: action.payload }
                }
            });
        case atomicActions_1.ActionType.SET_OVERRIDE_X_DOMAIN:
            if (action.payload) {
                return update(state, {
                    uiStateConsumerOverrides: {
                        xDomain: { $set: action.payload }
                    }
                });
            }
            else {
                return update(state, {
                    uiStateConsumerOverrides: {
                        xDomain: { $set: null }
                    }
                });
            }
        case atomicActions_1.ActionType.SET_Y_DOMAINS:
            return update(state, {
                uiState: {
                    yDomainBySeriesId: { $set: action.payload }
                }
            });
        case atomicActions_1.ActionType.SET_OVERRIDE_Y_DOMAINS:
            if (action.payload) {
                return update(state, {
                    uiStateConsumerOverrides: {
                        yDomainBySeriesId: { $set: action.payload }
                    }
                });
            }
            else {
                return update(state, {
                    uiStateConsumerOverrides: {
                        yDomainBySeriesId: { $set: null }
                    }
                });
            }
        case atomicActions_1.ActionType.SET_HOVER:
            return update(state, {
                uiState: {
                    hover: { $set: action.payload }
                }
            });
        case atomicActions_1.ActionType.SET_OVERRIDE_HOVER:
            if (action.payload != null) {
                return update(state, {
                    uiStateConsumerOverrides: {
                        hover: { $set: action.payload }
                    }
                });
            }
            else {
                return update(state, {
                    uiStateConsumerOverrides: {
                        hover: { $set: null }
                    }
                });
            }
        case atomicActions_1.ActionType.SET_SELECTION:
            return update(state, {
                uiState: {
                    selection: { $set: action.payload }
                }
            });
        case atomicActions_1.ActionType.SET_OVERRIDE_SELECTION:
            if (action.payload) {
                return update(state, {
                    uiStateConsumerOverrides: {
                        selection: { $set: action.payload }
                    }
                });
            }
            else {
                return update(state, {
                    uiStateConsumerOverrides: {
                        selection: { $set: null }
                    }
                });
            }
        default:
            return state;
    }
}
exports.default = default_1;
//# sourceMappingURL=reducer.js.map