"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var redux_1 = require("redux");
var redux_thunk_1 = require("redux-thunk");
var redux_debounced_1 = require("redux-debounced");
var reducer_1 = require("./reducer");
// chartId is only used for memoization.
function _createStore(_chartId, debugHooks) {
    var middlewares = [
        redux_debounced_1.default(),
        redux_thunk_1.default
    ];
    if (debugHooks && debugHooks.middlewares) {
        middlewares = middlewares.concat(debugHooks.middlewares);
    }
    var enhancers = [
        redux_1.applyMiddleware.apply(void 0, middlewares)
    ];
    if (debugHooks && debugHooks.enhancers) {
        enhancers = enhancers.concat(debugHooks.enhancers);
    }
    // hacking typings for `compose` because the redux typings have typings for when
    // the `compose` method has 1, 2, 3, and 3 + more arguments but no typings for
    // when the first argument is a spread
    return redux_1.createStore(reducer_1.default, redux_1.compose.apply(void 0, enhancers));
}
var memoizedCreateStore = _.memoize(_createStore);
function default_1(chartId, debugHooks) {
    if (chartId) {
        return memoizedCreateStore(chartId, debugHooks);
    }
    else {
        return _createStore(chartId, debugHooks);
    }
}
exports.default = default_1;
//# sourceMappingURL=storeFactory.js.map