"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var classNames = require("classnames");
var d3Scale = require("d3-scale");
var LEFT_MOUSE_BUTTON = 0;
var MouseCapture = (function (_super) {
    __extends(MouseCapture, _super);
    function MouseCapture() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {};
        _this._addWindowMouseEventHandlers = function () {
            window.addEventListener('mousemove', _this._onMouseMoveInWindow);
            window.addEventListener('mouseup', _this._onMouseUpInWindow);
        };
        _this._removeWindowMouseEventHandlers = function () {
            window.removeEventListener('mousemove', _this._onMouseMoveInWindow);
            window.removeEventListener('mouseup', _this._onMouseUpInWindow);
        };
        _this._onMouseDownInCaptureArea = function (e) {
            if (e.button === LEFT_MOUSE_BUTTON) {
                _this.setState({
                    mouseDownClientX: e.clientX,
                    mouseDownClientY: e.clientY,
                    lastMouseMoveClientX: e.clientX,
                    lastMouseMoveClientY: e.clientY
                });
                if (_this.props.onDragStart) {
                    var _a = _this._createPhysicalToLogicalScales(), xScale = _a.xScale, yScale = _a.yScale;
                    _this.props.onDragStart(xScale(e.clientX), yScale(e.clientY), e);
                }
                _this._removeWindowMouseEventHandlers();
                _this._addWindowMouseEventHandlers();
            }
        };
        _this._onMouseMoveInCaptureArea = function (e) {
            if (_this.props.onHover) {
                var _a = _this._createPhysicalToLogicalScales(), xScale = _a.xScale, yScale = _a.yScale;
                _this.props.onHover(xScale(e.clientX), yScale(e.clientY), e);
            }
        };
        _this._onMouseUpInCaptureArea = function (e) {
            if (e.button === LEFT_MOUSE_BUTTON && _this.props.onClick && Math.abs(_this.state.mouseDownClientX - e.clientX) <= 2 && Math.abs(_this.state.mouseDownClientY - e.clientY) <= 2) {
                var _a = _this._createPhysicalToLogicalScales(), xScale = _a.xScale, yScale = _a.yScale;
                _this.props.onClick(xScale(e.clientX), yScale(e.clientY), e);
            }
        };
        _this._onMouseMoveInWindow = function (e) {
            _this._maybeDispatchDragHandler(e, _this.props.onDrag);
            _this.setState({
                lastMouseMoveClientX: e.clientX,
                lastMouseMoveClientY: e.clientY
            });
        };
        _this._onMouseUpInWindow = function (e) {
            _this._maybeDispatchDragHandler(e, _this.props.onDragEnd);
            _this._removeWindowMouseEventHandlers();
            _this._clearState();
        };
        _this._onMouseLeaveCaptureArea = function (e) {
            if (_this.props.onHover) {
                _this.props.onHover(undefined, undefined, e);
            }
        };
        _this._onWheel = function (e) {
            // In Chrome, shift + wheel results in horizontal scrolling and
            // deltaY == 0 while deltaX != 0, and deltaX should be used instead
            var delta = e.shiftKey ? e.deltaY || e.deltaX : e.deltaY;
            if (_this.props.onZoom && delta) {
                var zoomSpeed = typeof _this.props.zoomSpeed === 'function'
                    ? _this.props.zoomSpeed(e)
                    : _this.props.zoomSpeed;
                var zoomFactor = Math.exp(-delta * zoomSpeed);
                var _a = _this._createPhysicalToLogicalScales(), xScale = _a.xScale, yScale = _a.yScale;
                _this.props.onZoom(zoomFactor, xScale(e.clientX), yScale(e.clientY), e);
            }
        };
        return _this;
    }
    MouseCapture.prototype.componentWillUnmount = function () {
        this._removeWindowMouseEventHandlers();
    };
    MouseCapture.prototype.render = function () {
        var _this = this;
        return (React.createElement("div", { className: classNames('lc-mouse-capture', this.props.className), onMouseDown: this._onMouseDownInCaptureArea, onMouseMove: this._onMouseMoveInCaptureArea, onMouseUp: this._onMouseUpInCaptureArea, onMouseLeave: this._onMouseLeaveCaptureArea, onWheel: this._onWheel, ref: function (element) { _this.element = element; } }, this.props.children));
    };
    MouseCapture.prototype._createPhysicalToLogicalScales = function () {
        var _a = this.element.getBoundingClientRect(), left = _a.left, right = _a.right, top = _a.top, bottom = _a.bottom;
        return {
            xScale: d3Scale.scaleLinear()
                .domain([left, right])
                .range([0, 1]),
            yScale: d3Scale.scaleLinear()
                .domain([top, bottom])
                .range([0, 1])
        };
    };
    MouseCapture.prototype._clearState = function () {
        this.setState({
            mouseDownClientX: undefined,
            mouseDownClientY: undefined,
            lastMouseMoveClientX: undefined,
            lastMouseMoveClientY: undefined
        });
    };
    MouseCapture.prototype._maybeDispatchDragHandler = function (e, handler) {
        if (e.button === LEFT_MOUSE_BUTTON && handler && this.state.mouseDownClientX != null) {
            var _a = this._createPhysicalToLogicalScales(), xScale = _a.xScale, yScale = _a.yScale;
            handler(xScale(e.clientX), yScale(e.clientY), e);
        }
    };
    return MouseCapture;
}(React.PureComponent));
MouseCapture.propTypes = {
    className: React.PropTypes.string,
    zoomSpeed: React.PropTypes.oneOfType([
        React.PropTypes.number,
        React.PropTypes.func
    ]),
    onZoom: React.PropTypes.func,
    onDragStart: React.PropTypes.func,
    onDrag: React.PropTypes.func,
    onDragEnd: React.PropTypes.func,
    onHover: React.PropTypes.func,
    onClick: React.PropTypes.func,
    children: React.PropTypes.oneOfType([
        React.PropTypes.element,
        React.PropTypes.arrayOf(React.PropTypes.element)
    ])
};
MouseCapture.defaultProps = {
    zoomSpeed: 0.05
};
exports.default = MouseCapture;
//# sourceMappingURL=MouseCapture.js.map