"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
function NonReactRender(component) {
    var prototype = component.prototype;
    var oldDidMount = prototype.componentDidMount;
    prototype.componentDidMount = function () {
        if (!_.isFunction(this.nonReactRender)) {
            throw new Error(this.constructor.name + ' must implement a nonReactRender function to use the NonReactRender decorator');
        }
        this.__boundNonReactRender = function () {
            this.__lastRafRequest = null;
            this.nonReactRender();
        }.bind(this);
        this.__lastRafRequest = requestAnimationFrame(this.__boundNonReactRender);
        if (oldDidMount) {
            oldDidMount.call(this);
        }
    };
    var oldDidUpdate = prototype.componentDidUpdate;
    prototype.componentDidUpdate = function () {
        if (!this.__lastRafRequest) {
            this.__lastRafRequest = requestAnimationFrame(this.__boundNonReactRender);
        }
        if (oldDidUpdate) {
            oldDidUpdate.call(this);
        }
    };
    var oldWillUnmount = prototype.componentWillUnmount;
    prototype.componentWillUnmount = function () {
        cancelAnimationFrame(this.__lastRafRequest);
        if (oldWillUnmount) {
            oldWillUnmount.call(this);
        }
    };
}
exports.default = NonReactRender;
//# sourceMappingURL=NonReactRender.js.map