"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
var d3Scale = require("d3-scale");
function enforceIntervalBounds(interval, bounds) {
    if (!bounds) {
        return interval;
    }
    var extent = intervalExtent(interval);
    var boundsExtent = intervalExtent(bounds);
    if (extent > boundsExtent) {
        var halfExtentDiff = (extent - boundsExtent) / 2;
        return {
            min: bounds.min - halfExtentDiff,
            max: bounds.max + halfExtentDiff
        };
    }
    else if (interval.min < bounds.min) {
        return {
            min: bounds.min,
            max: bounds.min + extent
        };
    }
    else if (interval.max > bounds.max) {
        return {
            min: bounds.max - extent,
            max: bounds.max
        };
    }
    else {
        return interval;
    }
}
exports.enforceIntervalBounds = enforceIntervalBounds;
function enforceIntervalExtent(interval, minExtent, maxExtent) {
    var extent = intervalExtent(interval);
    if (minExtent != null && extent < minExtent) {
        var halfExtentDiff = (minExtent - extent) / 2;
        return {
            min: interval.min - halfExtentDiff,
            max: interval.max + halfExtentDiff
        };
    }
    else if (maxExtent != null && extent > maxExtent) {
        var halfExtentDiff = (extent - maxExtent) / 2;
        return {
            min: interval.min + halfExtentDiff,
            max: interval.max - halfExtentDiff
        };
    }
    else {
        return interval;
    }
}
exports.enforceIntervalExtent = enforceIntervalExtent;
function intervalExtent(interval) {
    return interval.max - interval.min;
}
exports.intervalExtent = intervalExtent;
function extendInterval(interval, factor) {
    var extent = intervalExtent(interval);
    return {
        min: interval.min - extent * factor,
        max: interval.max + extent * factor
    };
}
exports.extendInterval = extendInterval;
function roundInterval(interval) {
    return {
        min: Math.round(interval.min),
        max: Math.round(interval.max)
    };
}
exports.roundInterval = roundInterval;
function niceInterval(interval) {
    var nicedInterval = d3Scale.scaleLinear().domain([interval.min, interval.max]).nice().domain();
    return {
        min: nicedInterval[0],
        max: nicedInterval[1]
    };
}
exports.niceInterval = niceInterval;
function mergeIntervals(intervals, defaultInterval) {
    if (intervals.length === 0) {
        return defaultInterval || undefined;
    }
    else {
        return {
            min: _.min(_.map(intervals, 'min')),
            max: _.max(_.map(intervals, 'max'))
        };
    }
}
exports.mergeIntervals = mergeIntervals;
function intervalContains(maybeLargerInterval, maybeSmallerInterval) {
    return maybeLargerInterval.min <= maybeSmallerInterval.min && maybeLargerInterval.max >= maybeSmallerInterval.max;
}
exports.intervalContains = intervalContains;
function panInterval(interval, delta) {
    return {
        min: interval.min + delta,
        max: interval.max + delta
    };
}
exports.panInterval = panInterval;
function zoomInterval(interval, factor, anchorBias) {
    if (anchorBias === void 0) { anchorBias = 0.5; }
    var currentExtent = intervalExtent(interval);
    var targetExtent = currentExtent / factor;
    var extentDelta = targetExtent - currentExtent;
    return {
        min: interval.min - extentDelta * anchorBias,
        max: interval.max + extentDelta * (1 - anchorBias)
    };
}
exports.zoomInterval = zoomInterval;
//# sourceMappingURL=intervalUtils.js.map