"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var d3Scale = require("d3-scale");
var NonReactRender_1 = require("../decorators/NonReactRender");
var PixelRatioContext_1 = require("../decorators/PixelRatioContext");
var PollingResizingCanvasLayer_1 = require("./PollingResizingCanvasLayer");
var renderUtils_1 = require("../renderUtils");
var componentUtils_1 = require("../componentUtils");
var propTypes_1 = require("../propTypes");
var interfaces_1 = require("../interfaces");
var BucketedLineLayer = (function (_super) {
    __extends(BucketedLineLayer, _super);
    function BucketedLineLayer() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.nonReactRender = function () {
            var _a = _this.refs['canvasLayer'].resetCanvas(), width = _a.width, height = _a.height, context = _a.context;
            _renderCanvas(_this.props, width, height, context);
        };
        return _this;
    }
    BucketedLineLayer.prototype.render = function () {
        return React.createElement(PollingResizingCanvasLayer_1.default, { ref: 'canvasLayer', onSizeChange: this.nonReactRender, pixelRatio: this.context.pixelRatio });
    };
    return BucketedLineLayer;
}(React.PureComponent));
BucketedLineLayer.propTypes = {
    data: React.PropTypes.arrayOf(propTypes_1.default.bucketDatum).isRequired,
    xDomain: propTypes_1.default.interval.isRequired,
    yDomain: propTypes_1.default.interval.isRequired,
    yScale: React.PropTypes.func,
    color: React.PropTypes.string
};
BucketedLineLayer.defaultProps = {
    yScale: d3Scale.scaleLinear,
    color: '#444',
    joinType: interfaces_1.JoinType.DIRECT
};
BucketedLineLayer = __decorate([
    NonReactRender_1.default,
    PixelRatioContext_1.default
], BucketedLineLayer);
function clamp(value, min, max) {
    return Math.min(Math.max(min, value), max);
}
// Export for testing.
function _renderCanvas(props, width, height, context) {
    var _a = renderUtils_1.getIndexBoundsForSpanData(props.data, props.xDomain, 'minXValue', 'maxXValue'), firstIndex = _a.firstIndex, lastIndex = _a.lastIndex;
    if (firstIndex === lastIndex) {
        return;
    }
    // Don't use rangeRound -- it causes flicker as you pan/zoom because it doesn't consistently round in one direction.
    var xScale = d3Scale.scaleLinear()
        .domain([props.xDomain.min, props.xDomain.max])
        .range([0, width]);
    var yScale = props.yScale()
        .domain([props.yDomain.min, props.yDomain.max])
        .range([0, height]);
    var computedValuesForVisibleData = props.data
        .slice(firstIndex, lastIndex)
        .map(function (datum) {
        // TODO: Why is this ceiling'd? There must have been a reason...
        // I think this was to avoid jitter, but if you zoom really slowly when the rects
        // are small you can still see them jitter in their width...
        var minX = Math.ceil(xScale(datum.minXValue));
        var maxX = Math.max(Math.floor(xScale(datum.maxXValue)), minX + 1);
        var minY = Math.floor(yScale(datum.minYValue));
        var maxY = Math.max(Math.floor(yScale(datum.maxYValue)), minY + 1);
        return {
            minX: minX,
            maxX: maxX,
            minY: minY,
            maxY: maxY,
            firstY: clamp(Math.floor(yScale(datum.firstYValue)), minY, maxY - 1),
            lastY: clamp(Math.floor(yScale(datum.lastYValue)), minY, maxY - 1),
            width: maxX - minX,
            height: maxY - minY
        };
    });
    // Bars
    context.beginPath();
    for (var i = 0; i < computedValuesForVisibleData.length; ++i) {
        var computedValues = computedValuesForVisibleData[i];
        if (computedValues.width !== 1 || computedValues.height !== 1) {
            context.rect(computedValues.minX, height - computedValues.maxY, computedValues.width, computedValues.height);
        }
    }
    context.fillStyle = props.color;
    context.fill();
    // Lines
    context.translate(0.5, -0.5);
    context.beginPath();
    var firstComputedValues = computedValuesForVisibleData[0];
    context.moveTo(firstComputedValues.maxX - 1, height - firstComputedValues.lastY);
    for (var i = 1; i < computedValuesForVisibleData.length; ++i) {
        var computedValues = computedValuesForVisibleData[i];
        if (props.joinType === interfaces_1.JoinType.LEADING) {
            context.lineTo(computedValuesForVisibleData[i - 1].maxX - 1, height - computedValues.firstY);
        }
        else if (props.joinType === interfaces_1.JoinType.TRAILING) {
            context.lineTo(computedValues.minX, height - computedValuesForVisibleData[i - 1].lastY);
        }
        context.lineTo(computedValues.minX, height - computedValues.firstY);
        context.moveTo(computedValues.maxX - 1, height - computedValues.lastY);
    }
    context.strokeStyle = props.color;
    context.stroke();
}
exports._renderCanvas = _renderCanvas;
exports.default = componentUtils_1.wrapWithAnimatedYDomain(BucketedLineLayer);
//# sourceMappingURL=BucketedLineLayer.js.map