"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var propTypes_1 = require("../propTypes");
var MouseCapture_1 = require("../MouseCapture");
var LEFT_MOUSE_BUTTON = 0;
exports.DEFAULT_SHOULD_ZOOM = function () { return true; };
exports.DEFAULT_SHOULD_PAN = function (event) { return !event.shiftKey && event.button === LEFT_MOUSE_BUTTON; };
exports.DEFAULT_SHOULD_BRUSH = function (event) { return event.shiftKey && event.button === LEFT_MOUSE_BUTTON; };
exports.DEFAULT_SHOULD_HOVER = function () { return true; };
var InteractionCaptureLayer = (function (_super) {
    __extends(InteractionCaptureLayer, _super);
    function InteractionCaptureLayer() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.state = {
            isPanning: false,
            isBrushing: false
        };
        _this._onZoom = function (factor, xPct, _yPct, e) {
            if (_this.props.onZoom && _this.props.shouldZoom && _this.props.shouldZoom(e)) {
                e.preventDefault();
                _this.props.onZoom(factor, xPct);
            }
        };
        _this._onDragStart = function (xPct, _yPct, e) {
            if (_this.props.onPan && _this.props.shouldPan && _this.props.shouldPan(e)) {
                _this.setState({ isPanning: true, lastPanXPct: xPct });
            }
            else if (_this.props.onBrush && _this.props.shouldBrush && _this.props.shouldBrush(e)) {
                _this.setState({ isBrushing: true, startBrushXPct: xPct });
            }
        };
        _this._onDrag = function (xPct, yPct, e) {
            _this._dispatchPanAndBrushEvents(xPct, yPct, e);
        };
        _this._onDragEnd = function (xPct, yPct, e) {
            _this._dispatchPanAndBrushEvents(xPct, yPct, e);
            _this.setState({
                isPanning: false,
                isBrushing: false,
                lastPanXPct: undefined,
                startBrushXPct: undefined
            });
        };
        _this._onClick = function (_xPct, _yPct, e) {
            if (_this.props.onBrush && _this.props.shouldBrush && _this.props.shouldBrush(e)) {
                _this.props.onBrush(undefined);
            }
        };
        _this._onHover = function (xPct, _yPct, e) {
            if (_this.props.onHover && _this.props.shouldHover && _this.props.shouldHover(e)) {
                if (xPct != null) {
                    _this.props.onHover(_this._xPctToDomain(xPct));
                }
                else {
                    _this.props.onHover(undefined);
                }
            }
        };
        return _this;
    }
    InteractionCaptureLayer.prototype.render = function () {
        return (React.createElement(MouseCapture_1.default, { className: 'interaction-capture-layer', zoomSpeed: this.props.zoomSpeed, onZoom: this._onZoom, onDragStart: this._onDragStart, onDrag: this._onDrag, onDragEnd: this._onDragEnd, onClick: this._onClick, onHover: this._onHover }));
    };
    InteractionCaptureLayer.prototype._dispatchPanAndBrushEvents = function (xPct, _yPct, _e) {
        if (this.props.onPan && this.state.isPanning) {
            this.props.onPan(this._xPctToDomain(this.state.lastPanXPct) - this._xPctToDomain(xPct));
            this.setState({ lastPanXPct: xPct });
        }
        else if (this.props.onBrush && this.state.isBrushing) {
            var a = this._xPctToDomain(this.state.startBrushXPct);
            var b = this._xPctToDomain(xPct);
            this.props.onBrush({ min: Math.min(a, b), max: Math.max(a, b) });
        }
    };
    InteractionCaptureLayer.prototype._xPctToDomain = function (xPct) {
        return this.props.xDomain.min + (this.props.xDomain.max - this.props.xDomain.min) * xPct;
    };
    return InteractionCaptureLayer;
}(React.PureComponent));
InteractionCaptureLayer.propTypes = {
    shouldZoom: React.PropTypes.func,
    shouldPan: React.PropTypes.func,
    shouldBrush: React.PropTypes.func,
    shouldHover: React.PropTypes.func,
    onZoom: React.PropTypes.func,
    onPan: React.PropTypes.func,
    onBrush: React.PropTypes.func,
    onHover: React.PropTypes.func,
    xDomain: propTypes_1.default.interval.isRequired,
    zoomSpeed: React.PropTypes.number
};
InteractionCaptureLayer.defaultProps = {
    zoomSpeed: 0.05
};
exports.default = InteractionCaptureLayer;
//# sourceMappingURL=InteractionCaptureLayer.js.map