"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var d3Scale = require("d3-scale");
var NonReactRender_1 = require("../decorators/NonReactRender");
var PixelRatioContext_1 = require("../decorators/PixelRatioContext");
var PollingResizingCanvasLayer_1 = require("./PollingResizingCanvasLayer");
var renderUtils_1 = require("../renderUtils");
var componentUtils_1 = require("../componentUtils");
var propTypes_1 = require("../propTypes");
var interfaces_1 = require("../interfaces");
var LineLayer = (function (_super) {
    __extends(LineLayer, _super);
    function LineLayer() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.nonReactRender = function () {
            var _a = _this.refs['canvasLayer'].resetCanvas(), width = _a.width, height = _a.height, context = _a.context;
            _renderCanvas(_this.props, width, height, context);
        };
        return _this;
    }
    LineLayer.prototype.render = function () {
        return React.createElement(PollingResizingCanvasLayer_1.default, { ref: 'canvasLayer', onSizeChange: this.nonReactRender, pixelRatio: this.context.pixelRatio });
    };
    return LineLayer;
}(React.PureComponent));
LineLayer.propTypes = {
    data: React.PropTypes.arrayOf(propTypes_1.default.pointDatum).isRequired,
    xDomain: propTypes_1.default.interval.isRequired,
    yDomain: propTypes_1.default.interval.isRequired,
    yScale: React.PropTypes.func,
    color: React.PropTypes.string
};
LineLayer.defaultProps = {
    yScale: d3Scale.scaleLinear,
    color: 'rgba(0, 0, 0, 0.7)',
    joinType: interfaces_1.JoinType.DIRECT
};
LineLayer = __decorate([
    NonReactRender_1.default,
    PixelRatioContext_1.default
], LineLayer);
// Export for testing.
function _renderCanvas(props, width, height, context) {
    // Should we draw something if there is one data point?
    if (props.data.length < 2) {
        return;
    }
    var _a = renderUtils_1.getIndexBoundsForPointData(props.data, props.xDomain, 'xValue'), firstIndex = _a.firstIndex, lastIndex = _a.lastIndex;
    if (firstIndex === lastIndex) {
        return;
    }
    var xScale = d3Scale.scaleLinear()
        .domain([props.xDomain.min, props.xDomain.max])
        .rangeRound([0, width]);
    var yScale = props.yScale()
        .domain([props.yDomain.min, props.yDomain.max])
        .rangeRound([0, height]);
    context.translate(0.5, -0.5);
    context.beginPath();
    context.moveTo(xScale(props.data[firstIndex].xValue), height - yScale(props.data[firstIndex].yValue));
    for (var i = firstIndex + 1; i < lastIndex; ++i) {
        var xValue = xScale(props.data[i].xValue);
        var yValue = height - yScale(props.data[i].yValue);
        if (props.joinType === interfaces_1.JoinType.LEADING) {
            context.lineTo(xScale(props.data[i - 1].xValue), yValue);
        }
        else if (props.joinType === interfaces_1.JoinType.TRAILING) {
            context.lineTo(xValue, height - yScale(props.data[i - 1].yValue));
        }
        context.lineTo(xValue, yValue);
    }
    context.strokeStyle = props.color;
    context.stroke();
}
exports._renderCanvas = _renderCanvas;
exports.default = componentUtils_1.wrapWithAnimatedYDomain(LineLayer);
//# sourceMappingURL=LineLayer.js.map