"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var d3Scale = require("d3-scale");
var NonReactRender_1 = require("../decorators/NonReactRender");
var PixelRatioContext_1 = require("../decorators/PixelRatioContext");
var PollingResizingCanvasLayer_1 = require("./PollingResizingCanvasLayer");
var renderUtils_1 = require("../renderUtils");
var componentUtils_1 = require("../componentUtils");
var propTypes_1 = require("../propTypes");
var TWO_PI = Math.PI * 2;
var PointLayer = (function (_super) {
    __extends(PointLayer, _super);
    function PointLayer() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this.nonReactRender = function () {
            var _a = _this.refs['canvasLayer'].resetCanvas(), width = _a.width, height = _a.height, context = _a.context;
            _renderCanvas(_this.props, width, height, context);
        };
        return _this;
    }
    PointLayer.prototype.render = function () {
        return React.createElement(PollingResizingCanvasLayer_1.default, { ref: 'canvasLayer', onSizeChange: this.nonReactRender, pixelRatio: this.context.pixelRatio });
    };
    return PointLayer;
}(React.PureComponent));
PointLayer.propTypes = {
    data: React.PropTypes.arrayOf(propTypes_1.default.pointDatum).isRequired,
    xDomain: propTypes_1.default.interval.isRequired,
    yDomain: propTypes_1.default.interval.isRequired,
    yScale: React.PropTypes.func,
    color: React.PropTypes.string,
    radius: React.PropTypes.number,
    innerRadius: React.PropTypes.number
};
PointLayer.defaultProps = {
    yScale: d3Scale.scaleLinear,
    color: 'rgba(0, 0, 0, 0.7)',
    radius: 3,
    innerRadius: 0
};
PointLayer = __decorate([
    NonReactRender_1.default,
    PixelRatioContext_1.default
], PointLayer);
// Export for testing.
function _renderCanvas(props, width, height, context) {
    var _a = renderUtils_1.getIndexBoundsForPointData(props.data, props.xDomain, 'xValue'), firstIndex = _a.firstIndex, lastIndex = _a.lastIndex;
    if (firstIndex === lastIndex) {
        return;
    }
    var xScale = d3Scale.scaleLinear()
        .domain([props.xDomain.min, props.xDomain.max])
        .rangeRound([0, width]);
    var yScale = props.yScale()
        .domain([props.yDomain.min, props.yDomain.max])
        .rangeRound([0, height]);
    var isFilled = props.innerRadius === 0;
    var radius = isFilled ? props.radius : (props.radius - props.innerRadius) / 2 + props.innerRadius;
    context.lineWidth = props.radius - props.innerRadius;
    context.strokeStyle = props.color;
    context.fillStyle = props.color;
    if (isFilled) {
        context.beginPath();
    }
    for (var i = firstIndex; i < lastIndex; ++i) {
        var x = xScale(props.data[i].xValue);
        var y = height - yScale(props.data[i].yValue);
        // `fill` can be batched, but `stroke` can't (it draws  extraneous lines even with `moveTo`).
        // https://html.spec.whatwg.org/multipage/scripting.html#dom-context-2d-arc
        if (!isFilled) {
            context.beginPath();
            context.arc(x, y, radius, 0, TWO_PI);
            context.stroke();
        }
        else {
            context.moveTo(x, y);
            context.arc(x, y, radius, 0, TWO_PI);
        }
    }
    if (isFilled) {
        context.fill();
    }
}
exports._renderCanvas = _renderCanvas;
exports.default = componentUtils_1.wrapWithAnimatedYDomain(PointLayer);
//# sourceMappingURL=PointLayer.js.map