"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _ = require("lodash");
function adjustBounds(firstIndex, lastIndex, dataLength) {
    if (firstIndex === dataLength || lastIndex === 0) {
        // No data is visible!
        return { firstIndex: firstIndex, lastIndex: lastIndex };
    }
    else {
        // We want to include the previous and next data points so that e.g. lines drawn across the canvas
        // boundary still have somewhere to go.
        return {
            firstIndex: Math.max(0, firstIndex - 1),
            lastIndex: Math.min(dataLength, lastIndex + 1)
        };
    }
}
var LOWER_BOUND_SENTINEL = (function () { });
var UPPER_BOUND_SENTINEL = (function () { });
// Assumption: data is sorted by `xValuePath` acending.
function getIndexBoundsForPointData(data, xValueBounds, xValueAccessor) {
    var lowerBound;
    var upperBound;
    var accessor;
    if (_.isString(xValueAccessor)) {
        lowerBound = _.set({}, xValueAccessor, xValueBounds.min);
        upperBound = _.set({}, xValueAccessor, xValueBounds.max);
        accessor = xValueAccessor;
    }
    else {
        lowerBound = LOWER_BOUND_SENTINEL;
        upperBound = UPPER_BOUND_SENTINEL;
        accessor = function (value) {
            if (value === LOWER_BOUND_SENTINEL) {
                return xValueBounds.min;
            }
            else if (value === UPPER_BOUND_SENTINEL) {
                return xValueBounds.max;
            }
            else {
                return xValueAccessor(value);
            }
        };
    }
    var firstIndex = _.sortedIndexBy(data, lowerBound, accessor);
    var lastIndex = _.sortedLastIndexBy(data, upperBound, accessor);
    return adjustBounds(firstIndex, lastIndex, data.length);
}
exports.getIndexBoundsForPointData = getIndexBoundsForPointData;
// Assumption: data is sorted by `minXValuePath` ascending.
function getIndexBoundsForSpanData(data, xValueBounds, minXValueAccessor, maxXValueAccessor) {
    var upperBound;
    var upperBoundAccessor;
    // Note that this purposely mixes the min accessor/max value. Think about it.
    if (_.isString(minXValueAccessor)) {
        upperBound = _.set({}, minXValueAccessor, xValueBounds.max);
        upperBoundAccessor = minXValueAccessor;
    }
    else {
        upperBound = UPPER_BOUND_SENTINEL;
        upperBoundAccessor = function (value) {
            if (value === UPPER_BOUND_SENTINEL) {
                return xValueBounds.max;
            }
            else {
                return minXValueAccessor(value);
            }
        };
    }
    var lowerBoundAccessor = _.isString(maxXValueAccessor)
        ? function (value) { return _.get(value, maxXValueAccessor); }
        : maxXValueAccessor;
    // Also note that this is a loose bound -- there could be spans that start later and end earlier such that
    // they don't actually fit inside the bounds, but this still saves us work in the end.
    var lastIndex = _.sortedLastIndexBy(data, upperBound, upperBoundAccessor);
    var firstIndex;
    for (firstIndex = 0; firstIndex < lastIndex; ++firstIndex) {
        if (lowerBoundAccessor(data[firstIndex]) >= xValueBounds.min) {
            break;
        }
    }
    return adjustBounds(firstIndex, lastIndex, data.length);
}
exports.getIndexBoundsForSpanData = getIndexBoundsForSpanData;
var DEFAULT_TICK_AMOUNT = 5;
function computeTicks(scale, ticks, tickFormat) {
    var outputTicks;
    if (ticks) {
        if (_.isFunction(ticks)) {
            var _a = scale.domain(), min = _a[0], max = _a[1];
            var maybeOutputTicks = ticks({ min: min, max: max });
            if (_.isNumber(maybeOutputTicks)) {
                outputTicks = scale.ticks(maybeOutputTicks);
            }
            else {
                outputTicks = maybeOutputTicks;
            }
        }
        else if (_.isArray(ticks)) {
            outputTicks = ticks;
        }
        else if (_.isNumber(ticks)) {
            outputTicks = scale.ticks(ticks);
        }
        else {
            throw new Error('ticks must be a function, array or number');
        }
    }
    else {
        outputTicks = scale.ticks(DEFAULT_TICK_AMOUNT);
    }
    var format;
    if (_.isFunction(tickFormat)) {
        format = tickFormat;
    }
    else {
        var tickCount = _.isNumber(ticks) ? ticks : DEFAULT_TICK_AMOUNT;
        format = scale.tickFormat(tickCount, tickFormat);
    }
    return { ticks: outputTicks, format: format };
}
exports.computeTicks = computeTicks;
//# sourceMappingURL=renderUtils.js.map